/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.jmx;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;

import javax.management.MBeanServer;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.PetalsJmxApiFactory;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DuplicatedServiceException;
import org.ow2.petals.jmx.api.api.exception.MissingServiceException;
import org.ow2.petals.microkernel.api.communication.JMXService;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * This class is a manager of JMX server. 
 * <p>
 * It is used to get a connection to a JMX Server on a distant container.
 * </p>
 * 
 * @author ddesjardins - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = JMXService.class))
public class JMXServiceImpl implements JMXService {

    /**
     * Logger
     */
    protected LoggingUtil log;

    /**
     * Admin Attributes Fractal component
     */
    @Requires(name = "adminAtt", signature = org.objectweb.fractal.jmx.agent.AdminAttributes.class)
    protected AdminAttributes adminAttributes;

    /**
     * Configuration service fractal component
     */
    @Requires(name = "configuration")
    protected ConfigurationService configurationService;

    /**
     * Map of PEtALS JMX connections
     */
    private Map<String, JMXClient> petalsJmxClients = new ConcurrentHashMap<String, JMXClient>();

    /**
     * The local container name
     */
    private String containerName;

    /**
     * {@inheritDoc}
     */
    public JMXClient getJMXClient(final String containerName) throws PetalsException {
        this.log.start("Container Name : " + containerName);

        final JMXClient petalsJmxClient;
        if (this.petalsJmxClients.containsKey(containerName)) {
            petalsJmxClient = this.petalsJmxClients.get(containerName);
        } else {
            try {
                if (this.containerName.equals(containerName)) {
                    // Create a JMX connection to a remote JMX server
                    petalsJmxClient = PetalsJmxApiFactory.getInstance().createJMXClient(
                            this.getLocalJMXServer());
                } else {
                    // Create a JMX connection to a remote JMX server
                    final ContainerConfiguration containerConfiguration = this.configurationService
                            .getContainerConfiguration(containerName);
                    petalsJmxClient = PetalsJmxApiFactory.getInstance().createJMXClient(
                            containerConfiguration.getHost(),
                            containerConfiguration.getJmxRMIConnectorPort(),
                            containerConfiguration.getUser(), containerConfiguration.getPassword());
                }

                this.petalsJmxClients.put(containerName, petalsJmxClient);
            } catch (final ConnectionErrorException e) {
                this.petalsJmxClients.remove(containerName);
                throw new PetalsException(
                        "Failed to connect the the remote JMX service of the container '"
                                + containerName + "'");
            } catch (final DuplicatedServiceException e) {
                throw new PetalsException(e);
            } catch (final MissingServiceException e) {
                throw new PetalsException(e);
            }
        }

        this.log.end();
        return petalsJmxClient;
    }

    /**
     * {@inheritDoc}
     */
    public MBeanServer getLocalJMXServer() {
        return this.adminAttributes.getRawMBeanServer();
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IllegalLifeCycleException, DuplicatedServiceException,
            MissingServiceException {
        this.log = new LoggingUtil(Logger.getLogger(JMXService.COMPONENT_LOGGER_NAME));
        this.log.start();

        this.containerName = this.configurationService.getContainerConfiguration().getName();

        // Initialize the Petals JMX API implementation
        PetalsJmxApiFactory.getInstance();

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws IllegalLifeCycleException {
        this.log.start();

        for (String containerName : this.petalsJmxClients.keySet()) {
            try {
                this.log.debug("Try to disconnect JMX connection to container : " + containerName);
                this.petalsJmxClients.get(containerName).disconnect();
            } catch (ConnectionErrorException e) {
                this.log.debug(e);
            }
        }

        PetalsJmxApiFactory.close();

        this.log.end();
    }

}
