/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.jndi.client;

import java.util.Hashtable;

import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.Name;
import javax.naming.NameClassPair;
import javax.naming.NameParser;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.communication.jndi.agent.JNDIAgentService;
import org.ow2.petals.microkernel.communication.jndi.client.naming.NamingContextFactory;

/**
 * InitialContextDecorator that wrap a real InitialContext object. 
 * <p>
 * All method are delegated, except the close() method, which do nothing. This class allows
 * to shared an InitialContext to several users, preventing one of them to close
 * the context and block the other ones. Use the _close() method to close the
 * underlying InitialContext.
 * </p>
 *
 * @author Adrien Louis - EBM WebSourcing
 * @author Roland NAUDIN - EBM WebSourcing
 */
public class InitialContextDecorator extends InitialContext {

    /**
     * the JNDI intial context.
     */
    private InitialContext initialContext;

    /**
     * Logging utilities.
     */
    private LoggingUtil log;

    /**
     * Create an InitialContext that wrap the given one. the close() method is
     * inactive.
     *
     * @param env
     *            The environment values
     * @param log
     *            the logging utility
     * @param jndiAgentService
     *            The local JNDI Agent
     * @throws NamingException
     *             impossible to initialize the context
     */
    @SuppressWarnings("unchecked")
    public InitialContextDecorator(final Hashtable<String, Object> env,
        final LoggingUtil log, JNDIAgentService jndiAgentService)
        throws NamingException {

        if (JNDIAgentService.JNDI_PETALS_FACTORY.equals(
                env.get(Context.INITIAL_CONTEXT_FACTORY))
            && env.containsKey(JNDIAgentService.ENVIRONMENT_LOCAL_CONNECTION)) {
            // create a local connection to the JNDI agent; this decorator is the InitialContext itself
            this.myProps = (Hashtable<Object, Object>) env.clone();
            this.defaultInitCtx = new NamingContextFactory().getInitialContext(
                env, jndiAgentService);
        } else {
            this.initialContext = new InitialContext(env);
        }

        this.log = log;
    }

    /**
     * (non-Javadoc).
     *
     * @see javax.naming.InitialContext#addToEnvironment(java.lang.String,
     *      java.lang.Object)
     * @param propName
     *            the name
     * @param propVal
     *            the value
     * @throws NamingException
     *             impossible to add to environment
     * @return the associated object
     */
    @Override
    public Object addToEnvironment(final String propName, final Object propVal)
        throws NamingException {
        Object out;

        if (this.initialContext == null) {
            myProps.put(propName, propVal);
            out = this.defaultInitCtx.addToEnvironment(propName, propVal);
        } else {
            out = this.initialContext.addToEnvironment(propName, propVal);
        }

        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#bind(javax.naming.Name,
     *      java.lang.Object)
     */
    @Override
    public void bind(Name name, Object obj) throws NamingException {
        if (this.initialContext == null) {
            this.defaultInitCtx.bind(name, obj);
        } else {
            this.initialContext.bind(name, obj);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#bind(java.lang.String, java.lang.Object)
     */
    @Override
    public void bind(String name, Object obj) throws NamingException {
        if (this.initialContext == null) {
            this.defaultInitCtx.bind(name, obj);
        } else {
            this.initialContext.bind(name, obj);
        }
    }

    /*
     * This InitialContext can not be closed by the user if it is seen as a pure
     * InitialContext. This method does not close the InitialContext
     */
    @Override
    public void close() throws NamingException {

        // Do not close the initialContext, use the specific one
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#composeName(javax.naming.Name,
     *      javax.naming.Name)
     */
    @Override
    public Name composeName(Name name, Name prefix) throws NamingException {
        Name out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.composeName(name, prefix);
        } else {
            out = this.initialContext.composeName(name, prefix);
        }
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#composeName(java.lang.String,
     *      java.lang.String)
     */
    @Override
    public String composeName(String name, String prefix)
        throws NamingException {
        String out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.composeName(name, prefix);
        } else {
            out = this.initialContext.composeName(name, prefix);
        }
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#createSubcontext(javax.naming.Name)
     */
    @Override
    public Context createSubcontext(Name name) throws NamingException {
        Context out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.createSubcontext(name);
        } else {
            out = this.initialContext.createSubcontext(name);
        }
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#createSubcontext(java.lang.String)
     */
    @Override
    public Context createSubcontext(String name) throws NamingException {
        Context out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.createSubcontext(name);
        } else {
            out = this.initialContext.createSubcontext(name);
        }
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#destroySubcontext(javax.naming.Name)
     */
    @Override
    public void destroySubcontext(Name name) throws NamingException {
        if (this.initialContext == null) {
            this.defaultInitCtx.destroySubcontext(name);
        } else {
            this.initialContext.destroySubcontext(name);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#destroySubcontext(java.lang.String)
     */
    @Override
    public void destroySubcontext(String name) throws NamingException {
        if (this.initialContext == null) {
            this.defaultInitCtx.destroySubcontext(name);
        } else {
            this.initialContext.destroySubcontext(name);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#getEnvironment()
     */
    @Override
    public Hashtable<?, ?> getEnvironment() throws NamingException {
        Hashtable<?, ?> out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.getEnvironment();
        } else {
            out = this.initialContext.getEnvironment();
        }
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#getNameInNamespace()
     */
    @Override
    public String getNameInNamespace() throws NamingException {
        String out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.getNameInNamespace();
        } else {
            out = this.initialContext.getNameInNamespace();
        }
        return out;
    }

    /**
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#getNameParser(javax.naming.Name)
     */
    @Override
    public NameParser getNameParser(Name name) throws NamingException {
        NameParser out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.getNameParser(name);
        } else {
            out = this.initialContext.getNameParser(name);
        }
        return out;
    }

    /**
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#getNameParser(java.lang.String)
     */
    @Override
    public NameParser getNameParser(String name) throws NamingException {
        NameParser out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.getNameParser(name);
        } else {
            out = this.initialContext.getNameParser(name);
        }
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#list(javax.naming.Name)
     */
    @Override
    public NamingEnumeration<NameClassPair> list(Name name)
        throws NamingException {
        log.start("Name = " + name);

        NamingEnumeration<NameClassPair> out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.list(name);
        } else {
            out = this.initialContext.list(name);
        }

        log.end();
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#list(java.lang.String)
     */
    @Override
    public NamingEnumeration<NameClassPair> list(String name)
        throws NamingException {
        log.start("String = " + name);

        NamingEnumeration<NameClassPair> out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.list(name);
        } else {
            out = this.initialContext.list(name);
        }

        log.end();
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#listBindings(javax.naming.Name)
     */
    @Override
    public NamingEnumeration<Binding> listBindings(Name name)
        throws NamingException {
        log.start("Name = " + name);

        NamingEnumeration<Binding> out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.listBindings(name);
        } else {
            out = this.initialContext.listBindings(name);
        }

        log.end();
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#listBindings(java.lang.String)
     */
    @Override
    public NamingEnumeration<Binding> listBindings(String name)
        throws NamingException {
        this.log.start("String = " + name);

        NamingEnumeration<Binding> out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.listBindings(name);
        } else {
            out = this.initialContext.listBindings(name);
        }

        this.log.end();
        return out;
    }

    /**
     * (non-Javadoc).
     *
     * @see javax.naming.InitialContext#lookup(javax.naming.Name)
     */
    @Override
    public Object lookup(final Name name) throws NamingException {
        this.log.start("Name = " + name);

        Object out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.lookup(name);
        } else {
            out = this.initialContext.lookup(name);
        }

        this.log.end();
        return out;
    }

    /**
     * (non-Javadoc).
     *
     *
     * @see javax.naming.InitialContext#lookup(java.lang.String)
     */
    @Override
    public Object lookup(final String name) throws NamingException {
        this.log.start("String = " + name);

        Object out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.lookup(name);
        } else {
            out = this.initialContext.lookup(name);
        }

        this.log.end();
        return out;
    }

    /**
     * (non-Javadoc).
     *
     * @see javax.naming.InitialContext#lookupLink(javax.naming.Name)
     */
    @Override
    public Object lookupLink(Name name) throws NamingException {
        this.log.start("Name = " + name);

        Object out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.lookup(name);
        } else {
            out = this.initialContext.lookup(name);
        }

        this.log.end();
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#lookupLink(java.lang.String)
     */
    @Override
    public Object lookupLink(String name) throws NamingException {
        this.log.start("String = " + name);

        Object out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.lookup(name);
        } else {
            out = this.initialContext.lookup(name);
        }

        this.log.end();
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#rebind(javax.naming.Name,
     *      java.lang.Object)
     */
    @Override
    public void rebind(Name name, Object obj) throws NamingException {
        if (this.initialContext == null) {
            this.defaultInitCtx.rebind(name, obj);
        } else {
            this.initialContext.rebind(name, obj);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#rebind(java.lang.String,
     *      java.lang.Object)
     */
    @Override
    public void rebind(String name, Object obj) throws NamingException {
        if (this.initialContext == null) {
            this.defaultInitCtx.rebind(name, obj);
        } else {
            this.initialContext.rebind(name, obj);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#removeFromEnvironment(java.lang.String)
     */
    @Override
    public Object removeFromEnvironment(String propName) throws NamingException {
        Object out;
        if (this.initialContext == null) {
            out = this.defaultInitCtx.removeFromEnvironment(propName);
        } else {
            out = this.initialContext.removeFromEnvironment(propName);
        }
        return out;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#rename(javax.naming.Name,
     *      javax.naming.Name)
     */
    @Override
    public void rename(Name oldName, Name newName) throws NamingException {
        if (this.initialContext == null) {
            this.defaultInitCtx.rename(oldName, newName);
        } else {
            this.initialContext.rename(oldName, newName);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#rename(java.lang.String,
     *      java.lang.String)
     */
    @Override
    public void rename(String oldName, String newName) throws NamingException {
        if (this.initialContext == null) {
            this.defaultInitCtx.rename(oldName, newName);
        } else {
            this.initialContext.rename(oldName, newName);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#unbind(javax.naming.Name)
     */
    @Override
    public void unbind(Name name) throws NamingException {
        if (this.initialContext == null) {
            this.defaultInitCtx.unbind(name);
        } else {
            this.initialContext.unbind(name);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.naming.InitialContext#unbind(java.lang.String)
     */
    @Override
    public void unbind(String name) throws NamingException {
        if (this.initialContext == null) {
            this.defaultInitCtx.unbind(name);
        } else {
            this.initialContext.unbind(name);
        }
    }

    /**
     * Close the underlying InitialContext wrapped object.
     *
     * @throws NamingException
     */
    public void closeInitialContextDecorator() throws NamingException {
        if (this.initialContext == null) {
            this.defaultInitCtx.close();
        } else {
            this.initialContext.close();
        }
    }

}
