/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.container.lifecycle;

import javax.jbi.JBIException;
import javax.jbi.management.LifeCycleMBean;

import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * Implementation of the <code>LifeCycle</code> mbean. 
 * <p>
 * This lifecycle mbean sends notifications about changes of state.
 * </p>
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public abstract class AbstractLifeCycle implements LifeCycleMBean {

    protected Object activitySynchronizer = new Byte[0];

    protected boolean initialized;

    protected LoggingUtil log;

    protected String name;

    /**
     * Force the set of a state, ignoring eventual errors when changing of
     * states
     */
    protected boolean forcedState = false;

    private String state;

    private final Object stateLock = new Object();

    /**
     * Creates a new instance of {@link AbstractLifeCycle}
     * 
     */
    public AbstractLifeCycle() {
        super();
        this.state = UNKNOWN;
    }

    /**
     * 
     * @throws JBIException
     */
    public abstract void doInit() throws JBIException;

    /**
     * 
     * @throws JBIException
     */
    public abstract void doShutdown() throws JBIException;

    /**
     * 
     * @throws JBIException
     */
    public abstract void doStart() throws JBIException;

    /**
     * 
     * @throws JBIException
     */
    public abstract void doStop() throws JBIException;

    /**
     * @see javax.jbi.management.LifeCycleMBean#getCurrentState()
     */
    public String getCurrentState() {
        synchronized (this.stateLock) {
            if (!(SHUTDOWN.equals(this.state) || STARTED.equals(this.state)
                    || STOPPED.equals(this.state) || UNKNOWN.equals(this.state))) {
                return UNKNOWN;
            } else {
                return this.state;
            }
        }
    }

    public boolean isShutdownState() {
        return SHUTDOWN.equals(this.getCurrentState());
    }

    public boolean isStartedState() {
        return STARTED.equals(this.getCurrentState());
    }

    public boolean isStoppedState() {
        return STOPPED.equals(this.getCurrentState());
    }

    public boolean isUnknownState() {
        return UNKNOWN.equals(this.getCurrentState());
    }

    public void setForcedState(boolean forcedState) {
        this.forcedState = forcedState;
    }

    /**
     * Shutdown the component. A notification will be thrown in case of
     * runtimeException or incoherent state.
     * 
     * @see javax.jbi.management.LifeCycleMBean#shutdown()
     * @throws JBIException
     *             in case of runtimeException or incoherent state.
     */
    public void shutDown() throws JBIException {
        this.log.start();

        if (SHUTDOWN.equals(this.state)) {
            this.log.warning("The component '" + this.name + "' is already shut down");
        } else if (forcedState || STOPPED.equals(this.state)) {
            synchronized (this.activitySynchronizer) {
                try {
                    this.doShutdown();
                    this.setState(SHUTDOWN);
                    this.initialized = false;
                } catch (final Throwable re) {
                    if (re instanceof ForbiddenActionJBIException) {
                        this.log.warning(re.getMessage());
                    } else {
                        this.log.error(re.getMessage(), re);
                        this.setState(UNKNOWN);
                        final JBIException e = new JBIException(
                                "An exception occured while shutting down the "
                                        + this.getJBIArtifactType() + " '" + this.name + "' : "
                                        + re.getMessage());
                        throw e;
                    }
                }
            }
        } else {
            final JBIException e = new JBIException("The " + this.getJBIArtifactType() + " '"
                    + this.name + "' cannot be shut down in this state: " + this.state);
            throw e;
        }

        this.log.end();
    }

    /**
     * Start the component. A notification will be thrown in case of
     * runtimeException or incoherent state.
     * 
     * @see javax.jbi.management.LifeCycleMBean#start()
     * @throws JBIException
     *             in case of runtimeException or incoherent state.
     */
    public void start() throws JBIException {
        this.log.start();

        if (STARTED.equals(this.state)) {
            this.log.warning("The component '" + this.name + "' is already started");
        } else if (forcedState || SHUTDOWN.equals(this.state) || STOPPED.equals(this.state)) {
            synchronized (this.activitySynchronizer) {
                try {
                    if (!this.initialized) {
                        this.doInit();
                        this.initialized = true;
                    }
                    this.doStart();
                    this.setState(STARTED);
                } catch (final Throwable re) {
                    if (re instanceof ForbiddenActionJBIException) {
                        this.log.warning(re.getMessage());
                    } else {
                        this.log.error(re.getMessage(), re);
                        this.setState(UNKNOWN);
                        final JBIException e = new JBIException(
                                "An exception occured while starting the "
                                        + this.getJBIArtifactType() + " '" + this.name + "' : "
                                        + re.getMessage());
                        throw e;
                    }
                }
            }
        } else {
            final JBIException e = new JBIException("The " + this.getJBIArtifactType() + " '"
                    + this.name + "' cannot be started in this state: " + this.state);
            throw e;
        }

        this.log.end();
    }

    /**
     * Stop the component. A notification will be thrown in case of
     * runtimeException or incoherent state.
     * 
     * @see javax.jbi.management.LifeCycleMBean#stop()
     * @throws JBIException
     *             in case of runtimeException or incoherent state.
     */
    public void stop() throws JBIException {
        this.log.start();
        if (STOPPED.equals(this.state)) {
            this.log.warning("The component '" + this.name + "' is already stopped");
        } else if (forcedState || STARTED.equals(this.state)) {
            synchronized (this.activitySynchronizer) {
                try {
                    this.doStop();
                    this.setState(STOPPED);
                } catch (final Throwable re) {
                    if (re instanceof ForbiddenActionJBIException) {
                        this.log.warning(re.getMessage());
                    } else {
                        this.log.error(re.getMessage(), re);
                        this.setState(UNKNOWN);
                        final JBIException e = new JBIException(
                                "An exception occured while stopping the "
                                        + this.getJBIArtifactType() + " '" + this.name + "' : "
                                        + re.getMessage());
                        throw e;
                    }
                }
            }
        } else {
            throw new JBIException("The " + this.getJBIArtifactType() + " '" + this.name
                    + "' cannot be stopped in this state: " + this.state);
        }
        this.log.end();
    }

    protected abstract String getJBIArtifactType();

    /**
     * Set the <code>State</code> reference of the current state. An
     * <code>LifeCycleStateEventAbstract</code> is thrown.
     * 
     * @param state
     */
    protected void setState(String state) throws JBIException {
        if (!(SHUTDOWN.equals(state) || STARTED.equals(state) || STOPPED.equals(state) || UNKNOWN
                .equals(state))) {
            throw new JBIException("State '" + state
                    + "' isn't defined by the JBI specification. No state changement done.");
        }
        synchronized (this.stateLock) {
            this.state = state;
        }
    }
}
