/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.container.lifecycle;

import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.management.ComponentLifeCycleMBean;
import javax.management.ObjectName;
import javax.naming.InitialContext;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.launcher.api.service.ServiceEndpoint.EndpointType;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.container.ComponentLifeCycle;
import org.ow2.petals.microkernel.api.container.ServiceUnitLifeCycle;
import org.ow2.petals.microkernel.api.container.thread.ComponentLifeCycleThread;
import org.ow2.petals.microkernel.api.jbi.ComponentContext;
import org.ow2.petals.microkernel.api.jbi.management.AdminService;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.messaging.DeliveryChannel;
import org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.communication.jndi.client.JNDIService;
import org.ow2.petals.microkernel.jbi.messaging.endpoint.JBIServiceEndpointImpl;

/**
 * Implementation of the <code>ComponentLifeCycle</code> mbean.
 * @author Adrien LOUIS - EBM WebSourcing
 * @author wjoseph - EBM WebSourcing
 * @author Nicolas Salatge - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = ComponentLifeCycle.class))
public class ComponentLifeCycleImpl extends AbstractLifeCycle implements ComponentLifeCycle,
        ComponentLifeCycleMBean {

    /**
     * Platform component :: SystemState Service
     */
    @Requires(name = "systemstate")
    private SystemStateService systemStateService;

    /**
     * Communication component :: JNDIService
     */
    @Requires(name = "jndi")
    protected JNDIService jndiService;

    /**
     * JBI-Management Component :: Admin Service
     */
    @Requires(name = "admin")
    protected AdminService adminService;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration")
    private ConfigurationService configurationService;

    /**
     * The container configuration
     */
    private ContainerConfiguration containerConfiguration;

    /**
     * The component context
     */
    private ComponentContext componentContext;

    /**
     * The component life cycle thread. Each life cycle have a thread to process
     * operation in the component ClassLoader
     */
    protected ComponentLifeCycleThread componentLifeCycleThread;

    protected String installationRootPath;

    protected ServiceEndpoint address;

    protected InitialContext initialContext;

    /**
     * Creates a new instance of {@link ComponentLifeCycleImpl}
     * 
     */
    public ComponentLifeCycleImpl() {
        super();
    }

    /**
     * {@inheritDoc}
     */
    public void init(ComponentContext componentContext,
            ComponentLifeCycleThread componentLifeCycleThread, Context context) throws JBIException {
        this.log.start();

        this.componentContext = componentContext;

        this.name = context.getDescriptor().getComponent().getIdentification().getName();

        this.installationRootPath = context.getInstallUrl().getPath();

        this.address = new JBIServiceEndpointImpl();
        this.address.setType(EndpointType.CONSUMER);
        this.address.getLocation().setComponentName(this.name);
        this.address.getLocation().setContainerName(this.containerConfiguration.getName());
        this.address.getLocation().setSubdomainName(this.containerConfiguration.getSubdomainName());

        // life cycle thread
        this.componentLifeCycleThread = componentLifeCycleThread;

        // set the state to SHUTDOWN, as the JBI specifications specifies
        this.setState(SHUTDOWN);

        this.log.end();
    }

    @Override
    public void doInit() throws JBIException {
        this.log.start();

        this.componentLifeCycleThread.doInit(this.componentContext);

        this.log.end();
    }

    @Override
    public void doShutdown() throws JBIException {
        this.log.start();

        // if a service unit (on any state) need this component, you can't
        // shutdown the component
        // TODO: rnaudin: remove this when the component would handle properly
        // SU lifecycles
        for (ServiceUnitLifeCycle su : this.adminService
                .getServiceUnitsLifeCyclesForComponent(this.name)) {
            if (!forcedState && (STARTED.equals(su.getState()) || STOPPED.equals(su.getState()))) {
                throw new ForbiddenActionJBIException(
                        "Cannot shut down a component with SUs at state STARTED or STOPPED");
            }
        }
        this.componentLifeCycleThread.doShutdown();

        // close the Channel if required
        DeliveryChannel channel = this.componentContext.getDeliveryChannel();
        if (channel != null && channel.isOpened()) {
            channel.close();
        }

        this.log.end();
    }

    @Override
    public void doStart() throws JBIException {
        this.log.start();

        this.componentLifeCycleThread.doStart();
        // boolean isFound = false;

        // TODO: rnaudin: uncomment this when the component implementations
        // would
        // handle properly Sus lifecycle against component lifecycle.

        // in the case of a restart, we reset all the shutdown SU LifeCycle to
        // their SA Lifecycle state
        // Collection<ServiceAssemblyLifeCycle> saLifeCycles = this.adminService
        // .getServiceAssemblies().values();
        //
        // List<ServiceUnitLifeCycle> suLifeCycles = this.adminService
        // .getServiceUnitsLifeCyclesForComponent(this.name);
        // for (ServiceUnitLifeCycle suLifeCycle : suLifeCycles) {
        // if (SHUTDOWN.equals(suLifeCycle.getCurrentState())) {
        // for (ServiceAssemblyLifeCycle salc : saLifeCycles) {
        // List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnit();
        // for (ServiceUnit unit : sus) {
        // if
        // (suLifeCycle.getSuName().equals(unit.getIdentification().getName()))
        // {
        // isFound = true;
        // if (STARTED.equals(salc.getCurrentState())) {
        // suLifeCycle.init();
        // suLifeCycle.start();
        // } else if (STOPPED.equals(salc.getCurrentState())) {
        // suLifeCycle.init();
        // }
        // break;
        // }
        // }
        // if (isFound) {
        // isFound = false;
        // break;
        // }
        // }
        //
        // }
        // }

        this.log.end();
    }

    @Override
    public void doStop() throws JBIException {
        this.log.start();

        // if a service unit (on any state) need this component, you can't stop
        // the component
        // TODO: rnaudin: remove this when the component implementations would
        // handle properly Sus lifecycle against component lifecycle.
        for (ServiceUnitLifeCycle su : this.adminService
                .getServiceUnitsLifeCyclesForComponent(this.name)) {
            if (!forcedState && STARTED.equals(su.getState())) {
                throw new ForbiddenActionJBIException(
                        "Cannot stop a component with SUs at state STARTED");
            }
        }

        this.componentLifeCycleThread.doStop();

        this.log.end();
    }

    /**
     * @see javax.jbi.management.ComponentLifeCycleMBean#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() throws JBIException {
        return this.componentLifeCycleThread.getExtensionMBeanName();
    }

    /**
     * 
     * @return
     */
    public String getName() {
        return this.name;
    }

    /**
     * Get the logger
     * 
     * @return the logger
     */
    public Logger getLogger() {
        return this.log.getLogger();
    }

    @Override
    protected void setState(String state) throws JBIException {
        super.setState(state);
        try {
            this.systemStateService.updateComponentLifeCycleState(this.name, state);
        } catch (Exception e) {
            String msg = "Component state can't be persisted";
            this.log.error(msg, e);
            throw new JBIException(msg, e);
        }
    }

    /**
     * Initialization of the petals component
     * 
     * @throws PetalsException
     * @throws JBIException
     * 
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void startComponentLifeCycle() {
        this.log = new LoggingUtil(Logger.getLogger(ComponentLifeCycle.COMPONENT_LOGGER_NAME));
        this.log.start();

        this.containerConfiguration = this.configurationService.getContainerConfiguration();

        this.log.end();

    }

    /**
     * Stop of the Fractal component, means shutdown the life cycle.
     * 
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stopComponentLifeCycle() {
        this.log.start();

        this.setForcedState(true);

        if (STARTED.equals(this.getCurrentState())) {
            try {
                this.doStop();
                this.doShutdown();
            } catch (JBIException e) {
                this.log.error("Failed to stop the component '" + this.name + "'", e);
            }
        }

        if (STOPPED.equals(this.getCurrentState())) {
            try {
                this.doShutdown();
            } catch (JBIException e) {
                this.log.error("Failed to shut down the component '" + this.name + "'", e);
            }
        }

        try {
            this.componentLifeCycleThread.shutdownThread();
        } catch (JBIException e) {
            this.log.error("Failed to stop the life-cycle dedicated thread for the component '"
                    + this.name + "'", e);
        }

        this.log.end();
    }

    @Override
    protected String getJBIArtifactType() {
        return "Component";
    }
}
