/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.container.lifecycle;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi.SharedLibrary;
import org.ow2.petals.microkernel.api.container.SharedLibraryLifeCycle;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.system.classloader.ClassLoaderService;

/**
 * The <code>LifeCycle</code> implementation of the shared library.
 * @author Nicolas SALATGE - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = SharedLibraryLifeCycle.class))
public class SharedLibraryLifeCycleImpl implements SharedLibraryLifeCycle {

    /**
     * state flag: the shared library is installed.
     */
    public static final String INSTALLED = "Installed";

    /**
     * state flag: the shared library is uninstalled.
     */
    public static final String UNINSTALLED = "Uninstalled";

    /**
     * the state of the shared library.
     */
    private String state;

    /**
     * the log.
     */
    private LoggingUtil log;

    /**
     * the shared library.
     */
    private SharedLibrary sharedLibrary;

    private final List<String> componentNames = new ArrayList<String>();

    /**
     * the base urls.
     */
    private URL[] baseUrls;

    /**
     * the use parent first.
     */
    private boolean useParentFirst;

    /**
     * Platform Component :: Loader Service.
     */
    @Requires(name = "classloader")
    private ClassLoaderService loaderService;

    /**
     * The default constructor.
     */
    public SharedLibraryLifeCycleImpl() {
        super();
        this.state = UNINSTALLED;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle#init(org.ow2
     * .petals.jbi.descriptor.original.generated.Jbi.SharedLibrary,
     * java.net.URL[], boolean)
     */
    public void init(final SharedLibrary sharedLibrary, final URL[] baseUrls1,
            final boolean useParentFirst) {
        this.sharedLibrary = sharedLibrary;
        this.baseUrls = baseUrls1;
        this.useParentFirst = useParentFirst;
        this.state = UNINSTALLED;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle#isLoaded()
     */
    public boolean isLoaded() {
        return SharedLibraryLifeCycleImpl.INSTALLED.equals(this.state);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle#loadSharedLibrary
     * ()
     */
    public void loadSharedLibrary() throws PetalsException {
        this.log.start();
        final String slName = this.sharedLibrary.getIdentification().getName();
        final String slVersion = this.sharedLibrary.getVersion();
        final String classLoaderId = slName + "-" + slVersion;
        final List<String> classpathUrls = this.sharedLibrary.getSharedLibraryClassPath()
                .getPathElement();
        this.loaderService.createSharedLibraryClassLoader(classLoaderId, this.baseUrls,
                classpathUrls, this.useParentFirst);
        this.state = SharedLibraryLifeCycleImpl.INSTALLED;

        this.log.info("Shared library '" + slName + "' (" + slVersion + ") loaded.");
        this.log.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle#unLoadSharedLibrary
     * ()
     */
    public void unLoadSharedLibrary() {
        this.log.start();

        String classLoaderId = this.sharedLibrary.getIdentification().getName() + "-"
                + this.sharedLibrary.getVersion();
        this.loaderService.deleteClassLoader(classLoaderId);
        this.state = SharedLibraryLifeCycleImpl.UNINSTALLED;

        this.log.end();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle#registerComponent
     * (java.lang.String)
     */
    public void registerComponent(String componentName) {
        this.componentNames.add(componentName);

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle#unregisterComponent
     * (java.lang.String)
     */
    public void unregisterComponent(String componentName) {
        this.componentNames.remove(componentName);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.container.lifecycle.SharedLibraryLifeCycle#
     * getRegisteredComponents()
     */
    public List<String> getRegisteredComponents() {
        return this.componentNames;
    }

    /**
     * {@inheritDoc}
     */
    public String getState() {
        return this.state;
    }

    /**
     * Start of the shared library lifecycle.
     * 
     */
    @LifeCycle(on = LifeCycleType.START)
    public void start() {
        this.log = new LoggingUtil(
                java.util.logging.Logger.getLogger(SharedLibraryLifeCycle.COMPONENT_LOGGER_NAME));
        this.log.call();
    }

    /**
     * Stop of the shared library lifecycle.
     * 
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() {
        this.log.call();
    }
}
