/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.uninstall.sl;

import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.management.task.AbstractLoggableTask;
import org.ow2.petals.systemstate.SystemStateException;
import org.ow2.petals.systemstate.generated.SharedLibrary;

/**
 * This task removes the shared lib from the state holder file.
 * <p>
 * This task MUST be executed with a {@link Context} containing:
 * <ul>
 * <li>component name as entity name,</li>
 * </ul>
 * and returns following data in the {@link Context}:
 * <ul>
 * <li>the shared library state holder ({@link SharedLibrary}).</li>
 * </ul>
 * </p>
 * <p>
 * The task has got a robust approach : it tries to do all the subtasks and
 * doesn't have the undo capability.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public class RemoveSLStateHolderTask extends AbstractLoggableTask {

    private final SystemStateService recoveryService;

    public RemoveSLStateHolderTask(final LoggingUtil log, final SystemStateService recoveryService,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
        this.recoveryService = recoveryService;
    }

    public void execute(Context context) {
        String slName = context.getEntityName();
        String version = context.getEntityVersion();
        SharedLibrary sharedLibraryStateHolder = null;

        if (slName != null) {
            try {
                sharedLibraryStateHolder = this.recoveryService.deleteSharedLibraryStateHolder(
                        slName, version);
                context.setSharedLibraryStateHolder(sharedLibraryStateHolder);
            } catch (SystemStateException e) {
                this.log.error("The shared library '" + slName + "' with version '" + version
                        + "' state holder can not be removed", e);
            }
        }
    }

    public void undo(Context context) throws Exception {
        // TODO : recreate the state holder
    }

}
