/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.routing.module;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Queue;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentLinkedQueue;

import javax.jbi.messaging.MessageExchange.Role;
import javax.xml.namespace.QName;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.api.jbi.ComponentContext;
import org.ow2.petals.microkernel.api.jbi.messaging.RoutingException;
import org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.messaging.routing.module.flow.PriorityModifier;
import org.ow2.petals.microkernel.jbi.messaging.routing.monitoring.RouterMonitorFilter;
import org.ow2.petals.microkernel.transport.util.TransportSendContext;
import org.ow2.petals.regulation.RegulationBuilder;
import org.ow2.petals.regulation.RegulationException;
import org.ow2.petals.regulation.generated.Flow;
import org.ow2.petals.regulation.generated.Regulation;

/**
 * The Flow Filter Module.
 * <p>
 * The configured flows to filter are monitored by this module. Each time a
 * component attempts a send, it is countered as an effective send.
 * </p>
 * <p>
 * If the number of the component send attempts overflows the maximum accepted
 * in a window, the messages are pending until the next window, by using the
 * Priority Queues.
 * </p>
 * 
 * @author Roland Naudin - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = SenderModule.class))
public class FlowFilterModule implements SenderModule {

    /**
     * The period to update the exchange priority.
     */
    private static final long PRIORITY_UPDATE_PERIOD = 1000;

    /**
     * Logger wrapper.
     */
    private LoggingUtil log;
    /**
     * The Priority Modifier fractal component
     */
    @Requires(name = "prioritymodifier")
    private PriorityModifier priorityModifier;

    private Map<RouterMonitorFilter, _Filter> filters;

    private Timer filterTimer;

    /**
     * Inner class to update the priority of pending exchanges.
     * 
     * @author rnaudin
     * 
     */
    private class PriorityUpdater extends TimerTask {

        @Override
        public void run() {
            for (_Filter filter : FlowFilterModule.this.filters.values()) {
                synchronized (filter) {
                    filter.currentWindow++;
                    if (filter.currentWindow == filter.expectedWindow) {
                        filter.currentWindow = 0;
                        filter.messagesSent = 0;
                        while (filter.pendingExchanges.size() > 0
                                && filter.messagesSent < filter.expectedFilter) {
                            MessageExchangeWrapper exchangeDecorator = filter.pendingExchanges
                                    .poll();
                            try {
                                FlowFilterModule.this.priorityModifier.modifyPriority(
                                        exchangeDecorator, (short) 3);
                                FlowFilterModule.this.log
                                        .debug("Reassign priority for exchange with Id '"
                                                + exchangeDecorator.getExchangeId() + "'");
                                filter.messagesSent++;
                            } catch (RoutingException e) {
                                FlowFilterModule.this.log.warning(
                                        "Failed to reassign a priority to the exchange with Id '"
                                                + exchangeDecorator.getExchangeId()
                                                + "', maybe a timeout has occurred", e);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Inner structure.
     * 
     * @author rnaudin
     * 
     */
    private class _Filter {

        public int expectedFilter;

        public int messagesSent;

        /**
         * unbound queue
         */
        public Queue<MessageExchangeWrapper> pendingExchanges;

        public int expectedWindow;

        public int currentWindow;

        public _Filter(int expectedFilter, int expectedWindow) {
            this.expectedFilter = expectedFilter;
            this.expectedWindow = expectedWindow;
            this.pendingExchanges = new ConcurrentLinkedQueue<MessageExchangeWrapper>();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.messaging.routing.module.SenderModule#electEndpoints
     * (java.util.Map, org.ow2.petals.jbi.component.context.ComponentContext,
     * org.ow2.petals.jbi.messaging.exchange.MessageExchangeDecorator)
     */
    public void electEndpoints(Map<ServiceEndpoint, TransportSendContext> electedEndpoints,
            ComponentContext sourceComponentContext, MessageExchangeWrapper exchangeDecorator)
            throws RoutingException {

        if (Role.CONSUMER.equals(exchangeDecorator.getRole())
                && exchangeDecorator.getMessage(MessageExchangeImpl.IN_MSG) != null) {
            for (RouterMonitorFilter routerMonitorFilter : FlowFilterModule.this.filters.keySet()) {
                String endpointName = null;
                QName interfaceName = exchangeDecorator.getInterfaceName();
                QName serviceName = exchangeDecorator.getService();
                QName operationName = exchangeDecorator.getOperation();
                if (exchangeDecorator.getEndpoint() != null) {
                    endpointName = exchangeDecorator.getEndpoint().getEndpointName();
                    interfaceName = exchangeDecorator.getEndpoint().getInterfaces()[0];
                    serviceName = exchangeDecorator.getEndpoint().getServiceName();
                }

                if ((routerMonitorFilter.getInterfaceName() == null || (interfaceName != null && interfaceName
                        .equals(routerMonitorFilter.getInterfaceName())))
                        && (routerMonitorFilter.getServiceName() == null || (serviceName != null && serviceName
                                .equals(routerMonitorFilter.getServiceName())))
                        && (routerMonitorFilter.getEndpointName() == null || (endpointName != null && endpointName
                                .equals(routerMonitorFilter.getEndpointName())))
                        && (routerMonitorFilter.getOperationName() == null || (operationName != null && operationName
                                .equals(routerMonitorFilter.getOperationName())))) {
                    _Filter filter = FlowFilterModule.this.filters.get(routerMonitorFilter);
                    boolean toFilter = false;
                    synchronized (filter) {
                        if (filter.messagesSent == filter.expectedFilter) {
                            filter.pendingExchanges.add(exchangeDecorator);
                            toFilter = true;
                        } else {
                            filter.messagesSent++;
                        }
                    }

                    if (toFilter) {
                        exchangeDecorator.setProperty(
                                PriorityOrdererModule.PROPERTY_ROUTER_PRIORITY, "0");
                        this.log.debug("Filter the exchange with Id '"
                                + exchangeDecorator.getExchangeId() + "'");
                    }

                }
            }
        }
    }

    /**
     * Start the Fractal component
     * 
     * @throws RegulationException
     * 
     * @throws RoutingException
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws RegulationException {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME_FLOW_FILTER));
        this.log.start();

        this.filters = new HashMap<RouterMonitorFilter, _Filter>();
        this.filterTimer = new Timer("Router - Flow filter module - Filter");

        try {
            if (this.loadRegulationConfiguration()) {
                this.filterTimer.schedule(new PriorityUpdater(), PRIORITY_UPDATE_PERIOD,
                        PRIORITY_UPDATE_PERIOD);
            }
        } catch (RegulationException e) {
            this.log.error("Failed to start Priority Regulator service", e);
            throw e;
        }

        this.log.end();
    }

    /**
     * Stop the Fractal component
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();

        this.filterTimer.cancel();
    }

    /**
     * load the Regulation configuration.
     * 
     * @throws RegulationException
     */
    private boolean loadRegulationConfiguration() throws RegulationException {
        boolean result = false;
        final URL regulationUrl = this.getClass().getResource(
                "/" + FlowControllerModule.REGULATION_CONFIGURATION_FILE);

        if (regulationUrl != null) {
            final Regulation regulation = RegulationBuilder.createRegulation(regulationUrl
                    .getFile());
            for (Flow flow : regulation.getFlow()) {
                RouterMonitorFilter routerMonitorFilter = new RouterMonitorFilter(flow
                        .getInterface(), flow.getService(), flow.getEndpoint(),
                        flow.getOperation(), false);
                if (flow.getFilter() != null) {
                    this.filters.put(routerMonitorFilter, new _Filter(flow.getFilter().getValue(),
                            flow.getFilter().getWindow()));
                    this.log.info("Add a flow filter on " + routerMonitorFilter
                            + " with threshold " + flow.getFilter().getValue() + " on window "
                            + flow.getFilter().getWindow());
                }
            }
            result = true;
        }

        return result;
    }
}
