/**
 * Copyright (c) 2009-2012 Capgemini, 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.platform.nio.server;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.transport.Transporter;
import org.ow2.petals.microkernel.transport.platform.nio.selector.InputSocketChannelContext;
import org.ow2.petals.microkernel.transport.platform.nio.selector.NioSelector;
import org.ow2.petals.microkernel.transport.platform.nio.selector.NioServer;
import org.ow2.petals.microkernel.transport.platform.nio.selector.PipedDeserializer;

/**
 * The server agent of the NIO transporter.
 * @author Christophe DENEUX - Capgemini Sud
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = NioServer.class))
public class NioServerAgent implements NioServer {

    /**
     * The socket selector Fractal component to use to process network IO.
     */
    @Requires(name = NioSelector.NIO_SELECTOR_ITF)
    protected NioSelector selectorAgent;

    @Requires(name = NioReceiver.NIO_MSG_RECEIVER_ITF)
    protected NioReceiver messageReceiver;

    /**
     * Interface used to access configuration
     */
    @Requires(name = "configuration")
    protected ConfigurationService configurationService;

    private LoggingUtil log;

    /**
     * The server socket on which connections are waited.
     */
    private InetSocketAddress serverSocket;

    /**
     * The server socket channel on which connections are waited.
     */
    private ServerSocketChannel serverSocketChannel;

    /**
     * Map containing the deserialization task associated to each socket
     * channel.
     */
    private final Map<SocketChannel, PipedDeserializer> deserializationTasks = new ConcurrentHashMap<SocketChannel, PipedDeserializer>();

    /**
     * Acceptor thread pool. An acceptor is in charge to deserialize received
     * data into its associated object and transmit it to the
     * {@link Transporter}.
     */
    private ThreadPoolExecutor threadPoolTaskExecutor = null;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IOException {

        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME));
        this.log.start();

        final int listenPort = this.configurationService.getContainerConfiguration().getTCPPort();
        final String host = this.configurationService.getContainerConfiguration().getHost();
        this.log.debug("Starting NIO transporter server on port " + listenPort);
        this.serverSocket = new InetSocketAddress(host, listenPort);
        this.serverSocketChannel = ServerSocketChannel.open();

        // Caution, as the queue of the thread pool is a LinkedBlockingQueue,
        // the parameters maxPoolSize and keep alive time have no sense.
        // TODO: When using JDK6, we can use
        // ThreadPoolExecutor.allowCoreThreadTimeOut to decrease the number of
        // thread in the pool.
        final int nbAcceptors = this.configurationService.getContainerConfiguration().getTCPReceivers();
        this.threadPoolTaskExecutor = new ThreadPoolExecutor(nbAcceptors, nbAcceptors, 5,
                TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(),
                new DeserializerThreadFactory());

        // TODO: Set socket options
        // this.serverSocketChannel.socket().setPerformancePreferences(0, 1, 2);

        this.serverSocketChannel.socket().bind(this.serverSocket, 100);
        this.serverSocketChannel.configureBlocking(false);

        this.log.info("The NIO transporter server is ready to process request on port "
                + listenPort);

        this.selectorAgent.register(SelectionKey.OP_ACCEPT, this.serverSocketChannel, this);

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        this.log.start();

        this.log.debug("Stopping NIO Server agent.");
        this.serverSocketChannel.close();
        this.log.info("The NIO Server agent is stopped.");

        this.threadPoolTaskExecutor.shutdownNow();

        this.log.end();
    }

    public void onConnectionIsAccepted(final SocketChannel socketChannel, final Selector selector)
            throws IOException {

        this.log.info("A connection is accepted: " + socketChannel.socket().toString());

        final PipedDeserializer deserializer = new PipedDeserializerImpl(socketChannel,
                this.messageReceiver, this.log.getLogger(), this.log);
        this.deserializationTasks.put(socketChannel, deserializer);

        socketChannel.register(selector, SelectionKey.OP_READ, new InputSocketChannelContext(
                socketChannel, deserializer, this));
    }

    public void onRead(final PipedDeserializer deserializer) {

        try {
            this.threadPoolTaskExecutor.execute(deserializer);
        } catch (final RejectedExecutionException e) {
            this.log.error("A message deserialization task has been rejected (" + e.getMessage()
                    + ")");
        }
    }

    public void onClose(final SocketChannel socketChannel) {
        // We free the deserializer associated to the socket channel

        this.deserializationTasks.remove(socketChannel);

    }
}
