/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.util;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;

import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.api.jbi.messaging.DeliveryChannel;

/**
 * @author Roland NAUDIN - EBM WebSourcing
 */
public final class TransporterUtil {

    public static final String PROPERTY_SENDSYNC_PROVIDER = "org.ow2.petals.messaging.sendSync.provider";

    public static final String PROPERTY_SENDSYNC_CONSUMER = "org.ow2.petals.messaging.sendSync.consumer";

    public static final String PROPERTY_OVERSENDSYNC = "org.ow2.petals.messaging.overSendSync";

    /**
     * Set the properties to the message exchange to specify that a synchronous
     * response is expected
     * 
     * @param exchange
     * @param overSyncExchange
     * 
     * @return {@code true} if the it is an 'overSync' exchange
     * 
     * @throws MessagingException
     *             The <code>sync_send()</code> is forbidden for acknowledgment
     *             send
     */
    public final static boolean setSendSyncProperties(
            final MessageExchangeWrapper exchangeDecorator) throws MessagingException {
        boolean result = false;

        if (!exchangeDecorator.getStatus().equals(ExchangeStatus.ACTIVE)) {
            throw new MessagingException(
                    "Synchronous send is forbidden when sending an acknowledgement");
        }

        // specify the role of the requester of the sync exchange, used
        // internally
        if (MessageExchange.Role.PROVIDER.equals(exchangeDecorator.getRole())) {
            if ((exchangeDecorator.getProperty(PROPERTY_SENDSYNC_PROVIDER) != null)
                    && exchangeDecorator.getProperty(PROPERTY_SENDSYNC_PROVIDER).equals(
                            Boolean.TRUE.toString())) {
                // set oversync flag
                exchangeDecorator.setProperty(PROPERTY_OVERSENDSYNC, Boolean.TRUE.toString());
                result = true;
            }
            exchangeDecorator.setProperty(PROPERTY_SENDSYNC_CONSUMER, Boolean.TRUE.toString());
        } else {
            if ((exchangeDecorator.getProperty(PROPERTY_SENDSYNC_CONSUMER) != null)
                    && exchangeDecorator.getProperty(PROPERTY_SENDSYNC_CONSUMER).equals(
                            Boolean.TRUE.toString())) {
                // set oversync flag
                exchangeDecorator.setProperty(PROPERTY_OVERSENDSYNC, Boolean.TRUE.toString());
                result = true;
            }
            exchangeDecorator.setProperty(PROPERTY_SENDSYNC_PROVIDER, Boolean.TRUE.toString());
        }

        // set the property as defined in the JBI specifications
        exchangeDecorator.setProperty(DeliveryChannel.PROPERTY_SENDSYNC, Boolean.TRUE.toString());

        return result;
    }

    /**
     * Update the sync properties of the message exchange
     * 
     * @param exchange
     * @return {@code true} if the it is an 'overSync' exchange
     */
    public final static boolean updateSyncProperties(final MessageExchangeWrapper exchangeDecorator) {
        boolean result = false;
        // remove the SENDSYNC property for eventually further steps of the
        // exchange
        if (exchangeDecorator.getProperty(PROPERTY_OVERSENDSYNC) != null) {
            exchangeDecorator.setProperty(PROPERTY_OVERSENDSYNC, null);
            result = true;
        } else {
            exchangeDecorator.setProperty(DeliveryChannel.PROPERTY_SENDSYNC, null);
        }
        if (MessageExchange.Role.PROVIDER.equals(exchangeDecorator.getRole())) {
            exchangeDecorator.setProperty(PROPERTY_SENDSYNC_PROVIDER, null);
        } else {
            exchangeDecorator.setProperty(PROPERTY_SENDSYNC_CONSUMER, null);
        }

        return result;
    }

    /**
     * Return the synchronized mode related to a message exchange.
     * 
     * @param messageExchange
     *            the message exchange to check
     * @param isResponse
     *            {@code true} if it is a check for a response message, {@code
     *            false} otherwise
     * @return true if message is in CONSUMER role and it is a response or if
     *         message is in PROVIDER role and it isn't a response, {@code
     *         false} otherwise
     */
    public final static boolean getSyncMode(final MessageExchangeWrapper exchangeDecorator,
            final boolean isResponse) {
        final boolean syncMode;
        if ((MessageExchange.Role.CONSUMER.equals(exchangeDecorator.getRole()) && isResponse)
                || (MessageExchange.Role.PROVIDER.equals(exchangeDecorator.getRole()) && !isResponse)) {
            syncMode = Boolean.parseBoolean((String) exchangeDecorator
                    .getProperty(TransporterUtil.PROPERTY_SENDSYNC_CONSUMER));
        } else {
            syncMode = Boolean.parseBoolean((String) exchangeDecorator
                    .getProperty(TransporterUtil.PROPERTY_SENDSYNC_PROVIDER));
        }
        return syncMode;
    }
}
