/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.download;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotSame;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.logging.Logger;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.management.task.DownloadTask;
import org.ow2.petals.microkernel.jbi.management.task.download.jetty.server.JettyServer;

/**
 * @author Christophe Deneux - EBM WebSourcing
 */
public class DownloadTaskTest {
    private JettyServer jettyServer = null;

    private String componentName = "test-contentType";

    @Rule
    public TemporaryFolder testFolder = new TemporaryFolder();

    @Before
    public void setUp() throws Exception {
        try {
            jettyServer = JettyServer.getInstance();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @After
    public void tearDown() throws Exception {
        try {
            jettyServer.stop();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testDownloadEntityPackageWithCorrectContentType() throws Exception {
        String urlTest = "http://localhost:8085/CorrectContentType/test-contentType.zip";

        final Context ctx = createContext(urlTest);

        final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
        containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                .getAbsolutePath());

        DownloadTask d = new DownloadTask(new LoggingUtil(Logger.getLogger("testLogger")),
                containerConfigurationMock);

        assertEquals(urlTest, ctx.getArchiveUrl().toString());

        d.execute(ctx);

        assertNotSame(ctx.getArchiveUrl().toString(), urlTest);

        boolean t = new File(ctx.getArchiveUrl().toURI()).exists();
        assertEquals(true, t);

    }

    @Test
    public void testDownloadEntityPackageWithWrongContentType() throws Exception {

        try {
            String test = "http://localhost:8085/WrongContentType/test-contentType.zip";

            final Context ctx = createContext(test);

            final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
            containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                    .getAbsolutePath());

            DownloadTask d = new DownloadTask(new LoggingUtil(Logger.getLogger("testLogger")),
                    containerConfigurationMock);

            d.execute(ctx);
        } catch (PetalsException pE) {
            assertTrue(pE.getMessage().contains(
                    "http://localhost:8085/WrongContentType/test-contentType.zip"));
            assertTrue(pE.getMessage().contains("application/toto"));
        }

    }
    
    @Test
    public void testDownloadEntityPackageWithCorrectContentTypeWithCharset() throws Exception {
        String urlTest = "http://localhost:8085/CorrectContentTypeWithCharset/test-contentType.zip";

        final Context ctx = createContext(urlTest);

        final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
        containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                .getAbsolutePath());

        DownloadTask d = new DownloadTask(new LoggingUtil(Logger.getLogger("testLogger")),
                containerConfigurationMock);

        assertEquals(urlTest, ctx.getArchiveUrl().toString());

        d.execute(ctx);

        assertNotSame(ctx.getArchiveUrl().toString(), urlTest);

        boolean t = new File(ctx.getArchiveUrl().toURI()).exists();
        assertEquals(true, t);

    }
      
    @Test
    public void testDownloadEntityPackageWithoutContentType() throws Exception {

        try {
            String test = "http://localhost:8085/WithoutContentType/test-contentType.zip";

            final Context ctx = createContext(test);

            final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
            containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                    .getAbsolutePath());

            DownloadTask d = new DownloadTask(new LoggingUtil(Logger.getLogger("testLogger")),
                    containerConfigurationMock);

            d.execute(ctx);
        } catch (PetalsException pE) {
            assertTrue(pE.getMessage().contains(
                    "http://localhost:8085/WithoutContentType/test-contentType.zip"));
            assertTrue(pE.getMessage().contains("It's content-type is undefined."));
        }

    }

    @Test
    public void testDownloadEntityPackageWithWrongURL() throws Exception {

        try {
            String test = "http://localhost:8085/WrongURL/test-contentType.zip";

            final Context ctx = createContext(test);

            final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
            containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                    .getAbsolutePath());

            DownloadTask d = new DownloadTask(new LoggingUtil(Logger.getLogger("testLogger")),
                    containerConfigurationMock);

            d.execute(ctx);
        } catch (PetalsException pE) {
            assertTrue(pE.getMessage().contains(
                    "http://localhost:8085/WrongURL/test-contentType.zip"));
            assertTrue(pE.getMessage().contains("HTTP 404"));
        }

    }

    private Context createContext(String test) throws URISyntaxException, MalformedURLException {
        final Context ctx = new Context();
        ctx.setEntityName(componentName);
        URI u = new URI(test);
       
        ctx.setArchiveUrl(u.toURL());
        return ctx;
    }

}
