/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation;

import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.microkernel.api.util.ZipUtil;

/**
 * Tests for InstallationUtils.
 * @author Olivier Fabre - EBM WebSourcing
 */
public class InstallationUtilsTest {

    /**
     * Base directory of the project
     */
    private File baseDir;

    /**
     * Test resources directory of the project
     */
    private File testResourcesDir;

    /**
     * Directory where extracted file will be placed
     */
    private File extractedDir;

    /**
     * Set up the base directory for the test
     */
    @Before
    public void setUp() {
        String baseDirPath = this.getClass().getResource(".").getPath();
        baseDirPath = baseDirPath.substring(0, baseDirPath.indexOf("target"));
        baseDirPath = baseDirPath.substring(baseDirPath.indexOf(":") + 1);
        try {
            baseDir = new File(new URI("file:" + baseDirPath).normalize());
        } catch (URISyntaxException e) {
            e.printStackTrace();
        }

        testResourcesDir = new File(baseDir, "src" + File.separator + "test" + File.separator
                + "resources");

        extractedDir = new File(testResourcesDir, "extracted");
    }

    /**
     * Check classpath elements in a directory
     * 
     * @throws IOException
     */
    @Test
    public void testCheckClassPathElements() throws IOException {
        /*
         * Create Test package
         */
        extractedDir.delete();
        extractedDir.mkdir();
        ZipUtil.unzipArchive(new File(testResourcesDir, "test-ziputil.zip"), extractedDir);

        /*
         * Check classpath elements
         */
        List<String> classpathElements = new ArrayList<String>();
        classpathElements.add("Lib01.jar");
        classpathElements.add("Lib02.jar");
        classpathElements.add("META-INF/jbi.xml");

        try {
            InstallationUtils.checkClassPathElements(extractedDir, classpathElements);
        } catch (PetalsException e) {
            fail("Can't find all classpath elements");
        }

        FileUtils.forceDelete(extractedDir);
    }

    /**
     * Try to find an inexistant classpath
     * 
     * @throws IOException
     * 
     */
    @Test
    public void testCheckClassPathElements2() throws IOException {

        /*
         * Create Test package
         */
        extractedDir.delete();
        extractedDir.mkdir();
        ZipUtil.unzipArchive(new File(testResourcesDir, "test-ziputil.zip"), extractedDir);

        /*
         * Check classpath elements
         */
        List<String> classpathElements = new ArrayList<String>();
        classpathElements.add("toto.jar");

        try {
            InstallationUtils.checkClassPathElements(extractedDir, classpathElements);
            fail("Can find an inexistant classpath element");
        } catch (PetalsException e) {
            // Nothing to do it's ok
        }

        FileUtils.forceDelete(extractedDir);

    }

    /**
     * Try to find an empty classpath element
     * 
     * @throws IOException
     * 
     */
    @Test
    public void testCheckClassPathElements3() throws IOException {

        /*
         * Create Test package
         */
        extractedDir.delete();
        extractedDir.mkdir();
        ZipUtil.unzipArchive(new File(testResourcesDir, "test-ziputil.zip"), extractedDir);

        /*
         * Check classpath elements
         */
        List<String> classpathElements = new ArrayList<String>();
        classpathElements.add("");

        try {
            InstallationUtils.checkClassPathElements(extractedDir, classpathElements);
            fail("Can find an empty classpath element");
        } catch (PetalsException e) {
            // Nothing to do it's ok
        }

        FileUtils.forceDelete(extractedDir);

    }

}
