/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.install.component;

import static org.junit.Assert.fail;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.logging.Logger;

import junit.framework.TestCase;

import org.easymock.EasyMock;
import org.junit.Test;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.ManagementException;
import org.ow2.petals.microkernel.api.jbi.management.Task;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.api.system.repository.exception.ComponentAlreadyExistsException;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.util.AbstractMavenTest;

/**
 * Test class for the task {@link CreateComponentRepositoryTask}.
 * @author Christophe DENEUX - EBM WebSourcing
 */
public class CreateComponentRepositoryTaskTest extends AbstractMavenTest {

    final static private String COMPONENT_NAME = "myComponentName";

    final static private String ARCHIVE_FILE_NAME = "archive.zip";

    final LoggingUtil log = new LoggingUtil(Logger.getLogger(
            CreateComponentRepositoryTaskTest.class.getName()));

    /**
     * Check interaction between the tested task and the repository service,
     * when processing the task.
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateComponentRepositoryTask_000() throws Exception {
        final RepositoryService repositorySvc = EasyMock.createMock(RepositoryService.class);
        EasyMock.expect(
                repositorySvc.addComponent(COMPONENT_NAME, new File(this.testResourcesDir,
                        ARCHIVE_FILE_NAME))).andReturn(
                new File(CreateComponentRepositoryTaskTest.this.targetDir, ARCHIVE_FILE_NAME));
        EasyMock.replay(repositorySvc);

        final Context ctx = new Context();
        ctx.setEntityName(COMPONENT_NAME);
        ctx.setUnzipRoot(new File(this.testResourcesDir, ARCHIVE_FILE_NAME));
        final Task task = new CreateComponentRepositoryTask(log, repositorySvc, null);

        // Process the task.
        task.execute(ctx);
        // Check returned value of the task.
        TestCase.assertEquals(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL(),
                ctx.getInstallUrl());

        // Verify interaction between the current test and the repository.
        EasyMock.verify(repositorySvc);
    }

    /**
     * Check interaction between the tested task and the repository service,
     * when processing the task twice for the same component.
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateComponentRepositoryTask_001() throws Exception {
        final RepositoryService repositorySvc = EasyMock.createMock(RepositoryService.class);
        EasyMock.expect(
                repositorySvc.addComponent(COMPONENT_NAME, new File(this.testResourcesDir,
                        ARCHIVE_FILE_NAME))).andReturn(
                new File(CreateComponentRepositoryTaskTest.this.targetDir, ARCHIVE_FILE_NAME));
        EasyMock.expect(
                repositorySvc.addComponent(COMPONENT_NAME, new File(this.testResourcesDir,
                        ARCHIVE_FILE_NAME))).andThrow(
                new ComponentAlreadyExistsException(COMPONENT_NAME, new Exception()));
        EasyMock.replay(repositorySvc);

        // First, process the task for the 1st installation of the component.
        {
            final Context ctx1 = new Context();
            ctx1.setEntityName(COMPONENT_NAME);
            ctx1.setUnzipRoot(new File(this.testResourcesDir, ARCHIVE_FILE_NAME));
            final Task task1 = new CreateComponentRepositoryTask(log, repositorySvc, null);
            task1.execute(ctx1);
            // Check returned value of the task.
            TestCase.assertEquals(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL(),
                    ctx1.getInstallUrl());
        }

        // Second, process the task for the 2nd installation of the component.
        try {
            final Context ctx2 = new Context();
            ctx2.setEntityName(COMPONENT_NAME);
            ctx2.setUnzipRoot(new File(this.testResourcesDir, ARCHIVE_FILE_NAME));
            final Task task2 = new CreateComponentRepositoryTask(log, repositorySvc, null);
            task2.execute(ctx2);
            fail("The exception " + ComponentAlreadyExistsException.class.getName()
                    + " is not raised.");
        } catch (final ManagementException e) {
            // This exception is expected.
            // We check the cause.
            TestCase.assertTrue(
                    ComponentAlreadyExistsException.class.getName() + " is not the cause of ManagementException",
                    e.getCause() instanceof ComponentAlreadyExistsException);
        }

        // Verify interaction between the current test and the repository
        // service.
        EasyMock.verify(repositorySvc);
    }

    /**
     * Check interaction between the tested task and the repository service,
     * when processing the task, next undoing it.
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateComponentRepositoryTask_002() throws Exception {
        final RepositoryService repositorySvc = EasyMock.createMock(RepositoryService.class);
        EasyMock.expect(
                repositorySvc.addComponent(COMPONENT_NAME, new File(this.testResourcesDir, ARCHIVE_FILE_NAME))).andReturn(new File(CreateComponentRepositoryTaskTest.this.targetDir, ARCHIVE_FILE_NAME));
        repositorySvc.removeComponent(COMPONENT_NAME);
        EasyMock.replay(repositorySvc);

        final Context ctx = new Context();
        ctx.setEntityName(COMPONENT_NAME);
        ctx.setUnzipRoot(new File(this.testResourcesDir, ARCHIVE_FILE_NAME));
        final Task task = new CreateComponentRepositoryTask(log, repositorySvc, null);

        // First, process the task.
        task.execute(ctx);
        // Check returned value of the task.
        TestCase.assertEquals(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL(),
                ctx.getInstallUrl());

        // Second, undo the task.
        task.undo(ctx);

        // Verify interaction between the current test and the repository.
        EasyMock.verify(repositorySvc);
    }

    /**
     * Check interaction between the tested task and the repository service,
     * when undoing the task without having processed it before.
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateComponentStateHolderTask_003() throws Exception {
        final RepositoryService repositorySvc = EasyMock.createMock(RepositoryService.class);
        repositorySvc.removeComponent(COMPONENT_NAME);
        EasyMock.expectLastCall().andThrow(new FileNotFoundException());
        EasyMock.replay(repositorySvc);

        final Context ctx = new Context();
        ctx.setEntityName(COMPONENT_NAME);
        ctx.setUnzipRoot(new File(this.testResourcesDir, ARCHIVE_FILE_NAME));
        final Task task = new CreateComponentRepositoryTask(log, repositorySvc, null);

        // Undo the task.
        task.undo(ctx);

        // No exception is raised if an error occurs during undoing.

        // Verify interaction between the current test and the repository.
        EasyMock.verify(repositorySvc);
    }

}
