/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.install.component;

import static org.junit.Assert.fail;

import java.io.File;
import java.util.logging.Logger;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.Task;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.system.systemstate.exception.ComponentAlreadyExistsException;
import org.ow2.petals.microkernel.util.AbstractMavenTest;
import org.ow2.petals.systemstate.generated.Component;

/**
 * Test class for the task {@link CreateComponentStateHolderTask}.
 * @author Christophe DENEUX - EBM WebSourcing
 */
public class CreateComponentStateHolderTaskTest extends AbstractMavenTest {

    final static private String COMPONENT_NAME = "myComponentName";

    final static private String ARCHIVE_FILE_NAME = "archive.zip";

    final LoggingUtil log = new LoggingUtil(Logger.getLogger(
            CreateComponentStateHolderTaskTest.class.getName()));

    @Rule
    public TemporaryFolder testFolder = new TemporaryFolder();

    @Before
    public void setUp() throws Exception {
        super.setUp();
    }

    /**
     * Check interaction between the tested task and the system state service,
     * when processing the task.
     * 
     * @throws Exception
     */
    @Test
    public void testCreateComponentStateHolderTask_000() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        EasyMock.expect(
                systemStateSvc.createComponentStateHolder(COMPONENT_NAME, this.targetDir.toURI()
                        .toURL(), ARCHIVE_FILE_NAME)).andAnswer(new IAnswer<Component>() {

            @Override
            public Component answer() throws Throwable {
                final Component component = new Component();
                component.setName(COMPONENT_NAME);
                return component;
            }
        });
        EasyMock.replay(systemStateSvc);

        final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
        containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                .getAbsolutePath());

        final Context ctx = new Context();
        ctx.setEntityName(COMPONENT_NAME);
        ctx.setArchiveUrl(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL());
        ctx.setInstallUrl(this.targetDir.toURI().toURL());
        final Task task = new CreateComponentStateHolderTask(log, systemStateSvc,
                containerConfigurationMock);

        // Process the task.
        task.execute(ctx);

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);
    }

    /**
     * Check interaction between the tested task and the system state service,
     * when processing the task twice for the same component.
     * 
     * @throws Exception
     */
    @Test
    public void testCreateComponentStateHolderTask_001() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        EasyMock.expect(
                systemStateSvc.createComponentStateHolder(COMPONENT_NAME, this.targetDir.toURI()
                        .toURL(), ARCHIVE_FILE_NAME)).andAnswer(new IAnswer<Component>() {

            @Override
            public Component answer() throws Throwable {
                final Component component = new Component();
                component.setName(COMPONENT_NAME);
                return component;
            }
        });
        EasyMock.expect(
                systemStateSvc.createComponentStateHolder(COMPONENT_NAME, this.targetDir.toURI()
                        .toURL(), ARCHIVE_FILE_NAME)).andThrow(
                new ComponentAlreadyExistsException(COMPONENT_NAME));
        EasyMock.replay(systemStateSvc);

        final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
        containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                .getAbsolutePath());

        // First, process the task for the 1st installation of the component.
        {
            final Context ctx1 = new Context();
            ctx1.setEntityName(COMPONENT_NAME);
            ctx1.setArchiveUrl(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL());
            ctx1.setInstallUrl(this.targetDir.toURI().toURL());
            final Task task1 = new CreateComponentStateHolderTask(log, systemStateSvc,
                    containerConfigurationMock);

            task1.execute(ctx1);
        }

        // Second, process the task for the 2nd installation of the component.
        try {
            final Context ctx2 = new Context();
            ctx2.setEntityName(COMPONENT_NAME);
            ctx2.setArchiveUrl(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL());
            ctx2.setInstallUrl(this.targetDir.toURI().toURL());
            final Task task2 = new CreateComponentStateHolderTask(log, systemStateSvc,
                    containerConfigurationMock);

            task2.execute(ctx2);
            fail("The exception " + ComponentAlreadyExistsException.class.getName()
                    + " is not raised.");
        } catch (final ComponentAlreadyExistsException e) {
            // This exception is expected.
        }

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);

    }

    /**
     * Check interaction between the tested task and the system state service,
     * when processing the task, next undoing it.
     * 
     * @throws Exception
     */
    @Test
    public void testCreateComponentStateHolderTask_002() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        final Component component = new Component();
        component.setName(COMPONENT_NAME);
        EasyMock.expect(
                systemStateSvc.createComponentStateHolder(COMPONENT_NAME, this.targetDir.toURI()
                        .toURL(), ARCHIVE_FILE_NAME)).andAnswer(new IAnswer<Component>() {

            @Override
            public Component answer() throws Throwable {
                return component;
            }
        });
        EasyMock.expect(systemStateSvc.deleteComponentStateHolder(COMPONENT_NAME)).andAnswer(
                new IAnswer<Component>() {

                    @Override
                    public Component answer() throws Throwable {
                        return component;
                    }
                });
        EasyMock.replay(systemStateSvc);

        final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
        containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                .getAbsolutePath());

        final Context ctx = new Context();
        ctx.setEntityName(COMPONENT_NAME);
        ctx.setArchiveUrl(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL());
        ctx.setInstallUrl(this.targetDir.toURI().toURL());
        final Task task = new CreateComponentStateHolderTask(log, systemStateSvc,
                containerConfigurationMock);

        // First, process the task.
        task.execute(ctx);

        // Second, undo the task.
        task.undo(ctx);

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);
    }

    /**
     * Check interaction between the tested task and the system state service,
     * when undoing the task without having processed it before.
     * 
     * @throws Exception
     */
    @Test
    public void testCreateComponentStateHolderTask_003() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        EasyMock.expect(systemStateSvc.deleteComponentStateHolder(COMPONENT_NAME)).andAnswer(
                new IAnswer<Component>() {

                    @Override
                    public Component answer() throws Throwable {
                        final Component component = new Component();
                        component.setName(COMPONENT_NAME);
                        return component;
                    }
                });
        EasyMock.replay(systemStateSvc);

        final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
        containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                .getAbsolutePath());

        final Context ctx = new Context();
        ctx.setEntityName(COMPONENT_NAME);
        ctx.setArchiveUrl(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL());
        ctx.setInstallUrl(this.targetDir.toURI().toURL());
        final Task task = new CreateComponentStateHolderTask(log, systemStateSvc,
                containerConfigurationMock);

        // Undo the task.
        task.undo(ctx);

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);
    }

}
