/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.install.sl;

import static org.junit.Assert.fail;

import java.io.File;
import java.util.logging.Logger;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.Task;
import org.ow2.petals.microkernel.api.system.SystemStateService;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.system.systemstate.exception.ComponentAlreadyExistsException;
import org.ow2.petals.microkernel.system.systemstate.exception.SharedLibraryAlreadyExistsException;
import org.ow2.petals.microkernel.util.AbstractMavenTest;
import org.ow2.petals.systemstate.generated.SharedLibrary;

/**
 * Test class for the task {@link CreateSLStateHolderTask}.
 * @author Christophe DENEUX - EBM WebSourcing
 */
public class CreateSLStateHolderTaskTest extends AbstractMavenTest {

    final static private String SL_NAME = "mySLName";

    final static private String SL_VERSION = "1.0.0";

    final static private String ARCHIVE_FILE_NAME = "archive.zip";

    final LoggingUtil log = new LoggingUtil(Logger.getLogger(
            CreateSLStateHolderTaskTest.class.getName()));

    @Rule
    public TemporaryFolder testFolder = new TemporaryFolder();

    @Before
    public void setUp() throws Exception {
        super.setUp();
    }

    /**
     * Check interaction between the tested task and the system state service,
     * when processing the task.
     * 
     * @throws Exception
     */
    @Test
    public void testCreateSLStateHolderTask_000() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        EasyMock.expect(
                systemStateSvc.createSharedLibraryStateHolder(SL_NAME, SL_VERSION, this.targetDir
                        .toURI().toURL(), ARCHIVE_FILE_NAME)).andAnswer(
                new IAnswer<SharedLibrary>() {

                    @Override
                    public SharedLibrary answer() throws Throwable {
                        final SharedLibrary sharedLibrary = new SharedLibrary();
                        sharedLibrary.setName(SL_NAME);
                        sharedLibrary.setVersion(SL_VERSION);
                        return sharedLibrary;
                    }
                });
        EasyMock.replay(systemStateSvc);

        final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
        containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                .getAbsolutePath());

        final Context ctx = new Context();
        ctx.setEntityName(SL_NAME);
        ctx.setEntityVersion(SL_VERSION);
        ctx.setArchiveUrl(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL());
        ctx.setInstallUrl(this.targetDir.toURI().toURL());
        final Task task = new CreateSLStateHolderTask(log, systemStateSvc,
                containerConfigurationMock);

        // Process the task.
        task.execute(ctx);

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);
    }

    /**
     * Check interaction between the tested task and the system state service,
     * when processing the task twice for the same component.
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateSLStateHolderTask_001() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        EasyMock.expect(
                systemStateSvc.createSharedLibraryStateHolder(SL_NAME, SL_VERSION, this.targetDir
                        .toURI().toURL(), ARCHIVE_FILE_NAME)).andAnswer(
                new IAnswer<SharedLibrary>() {

                    @Override
                    public SharedLibrary answer() throws Throwable {
                        final SharedLibrary sharedLibrary = new SharedLibrary();
                        sharedLibrary.setName(SL_NAME);
                        sharedLibrary.setVersion(SL_VERSION);
                        return sharedLibrary;
                    }
                });
        EasyMock.expect(
                systemStateSvc.createSharedLibraryStateHolder(SL_NAME, SL_VERSION, this.targetDir
                        .toURI().toURL(), ARCHIVE_FILE_NAME)).andThrow(
                new SharedLibraryAlreadyExistsException(SL_NAME, SL_VERSION));
        EasyMock.replay(systemStateSvc);

        final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
        containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                .getAbsolutePath());

        // First, process the task for the 1st installation of the shared
        // library.
        {
            final Context ctx1 = new Context();
            ctx1.setEntityName(SL_NAME);
            ctx1.setEntityVersion(SL_VERSION);
            ctx1.setArchiveUrl(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL());
            ctx1.setInstallUrl(this.targetDir.toURI().toURL());
            final Task task1 = new CreateSLStateHolderTask(log, systemStateSvc,
                    containerConfigurationMock);

            task1.execute(ctx1);
        }

        // Second, process the task for the 2nd installation of the shared
        // library.
        {
            try {
                final Context ctx2 = new Context();
                ctx2.setEntityName(SL_NAME);
                ctx2.setEntityVersion(SL_VERSION);
                ctx2.setArchiveUrl(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL());
                ctx2.setInstallUrl(this.targetDir.toURI().toURL());
                final Task task2 = new CreateSLStateHolderTask(log, systemStateSvc,
                        containerConfigurationMock);

                task2.execute(ctx2);
                fail("The exception " + ComponentAlreadyExistsException.class.getName()
                        + " is not raised.");
            } catch (final SharedLibraryAlreadyExistsException e) {
                // This exception is expected.
            }
        }

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);
    }

    /**
     * Check interaction between the tested task and the system state service,
     * when processing the task, next undoing it.
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateSLStateHolderTask_002() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        final SharedLibrary sharedLibrary = new SharedLibrary();
        sharedLibrary.setName(SL_NAME);
        sharedLibrary.setVersion(SL_VERSION);
        EasyMock.expect(
                systemStateSvc.createSharedLibraryStateHolder(SL_NAME, SL_VERSION, this.targetDir
                        .toURI().toURL(),
                        ARCHIVE_FILE_NAME)).andAnswer(new IAnswer<SharedLibrary>() {

            @Override
            public SharedLibrary answer() throws Throwable {
                        return sharedLibrary;
            }
        });
        EasyMock.expect(systemStateSvc.deleteSharedLibraryStateHolder(SL_NAME, SL_VERSION))
                .andAnswer(new IAnswer<SharedLibrary>() {

                    @Override
                    public SharedLibrary answer() throws Throwable {
                        return sharedLibrary;
                    }
                });
        EasyMock.replay(systemStateSvc);

        final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
        containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                .getAbsolutePath());

        final Context ctx = new Context();
        ctx.setEntityName(SL_NAME);
        ctx.setEntityVersion(SL_VERSION);
        ctx.setArchiveUrl(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL());
        ctx.setInstallUrl(this.targetDir.toURI().toURL());
        final Task task = new CreateSLStateHolderTask(log, systemStateSvc,
                containerConfigurationMock);

        // First, process the task.
        task.execute(ctx);

        // Second, undo the task.
        task.undo(ctx);

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);
    }

    /**
     * Check interaction between the tested task and the system state service,
     * when undoing the task without having processed it before.
     * 
     * @throws Exception
     */
    @Test    
    public void testCreateSLStateHolderTask_003() throws Exception {
        final SystemStateService systemStateSvc = EasyMock.createMock(SystemStateService.class);
        EasyMock.expect(systemStateSvc.deleteSharedLibraryStateHolder(SL_NAME, SL_VERSION))
                .andAnswer(
                new IAnswer<SharedLibrary>() {

                    @Override
                    public SharedLibrary answer() throws Throwable {
                        final SharedLibrary sharedLibrary = new SharedLibrary();
                        sharedLibrary.setName(SL_NAME);
                        sharedLibrary.setVersion(SL_VERSION);
                        return sharedLibrary;
                    }
                });
        EasyMock.replay(systemStateSvc);

        final ContainerConfiguration containerConfigurationMock = new ContainerConfiguration();
        containerConfigurationMock.setWorkDirectoryPath(this.testFolder.newFolder("work")
                .getAbsolutePath());

        final Context ctx = new Context();
        ctx.setEntityName(SL_NAME);
        ctx.setEntityVersion(SL_VERSION);
        ctx.setArchiveUrl(new File(this.targetDir, ARCHIVE_FILE_NAME).toURI().toURL());
        ctx.setInstallUrl(this.targetDir.toURI().toURL());
        final Task task = new CreateSLStateHolderTask(log, systemStateSvc,
                containerConfigurationMock);

        // Undo the task.
        task.undo(ctx);

        // Verify interaction between the current test and the system state.
        EasyMock.verify(systemStateSvc);
    }

}
