/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.exchange;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.net.URI;
import java.util.logging.Logger;

import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.RobustInOnly;
import javax.xml.namespace.QName;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfOperation;
import org.ow2.petals.commons.log.PetalsExecutionContext;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeCheckHelper;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint;

import com.ebmwebsourcing.easycommons.thread.ExecutionContext;

/**
 * @author Olivier Fabre - EBM WebSourcing
 */
public class PersistedContextualMessageExchangeFactoryImplTest extends MessageExchangeCheckHelper {

    protected final ServiceEndpoint consumerEndpointMock = Mockito.mock(ServiceEndpoint.class);

    protected final Logger logger = Logger.getLogger("testLogger");

    protected MessageExchangeFactory messageExchangeFactory;

    protected MessageExchangeFactory createTestMessageExchangeFactory() {
        return  PersistedContextualMessageExchangeFactoryImpl.createContextualMessageExchangeFactory(consumerEndpointMock, logger);
    }

    @Before
    public final void before() {
        messageExchangeFactory = createTestMessageExchangeFactory();
        PetalsExecutionContext.putContainerName("containerName");
    }

    @After
    public final void after() {
        ExecutionContext.getProperties().clear();
    }

    @Test
    public void testCreateExchangeFromServAndOp() throws MessagingException {
        QName serviceName = new QName("targetService");
        QName opName = new QName("targetOperation");

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(serviceName,
                opName);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
        assertGeneratedExchangeUseProvidedServiceAndOp(actualMessageExchange, serviceName, opName);
    }

    @Test
    public void testCreateExchangeFromLegacyInOnlyPattern() throws MessagingException {
        URI pattern = MessageExchangeImpl.IN_ONLY_PATTERN;

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOnly(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromNewInOnlyPattern() throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.IN_ONLY.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOnly(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromLegacyRobustInOnlyPattern() throws MessagingException {
        URI pattern = MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN;

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsRobustInOnly(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromNewRobustInOnlyPattern() throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.ROBUST_IN_ONLY.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsRobustInOnly(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromLegacyInOptOutPattern() throws MessagingException {
        URI pattern = MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN;

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOptOut(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromNewInOptOutPattern() throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.IN_OPTIONAL_OUT.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOptOut(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromLegacyInOutPattern() throws MessagingException {
        URI pattern = MessageExchangeImpl.IN_OUT_PATTERN;

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOut(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromNewInOutPattern() throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.IN_OUT.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOut(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromNonJBIPattern() {
        URI pattern = URI.create("http://non/jbi/pattern");

        try {
            messageExchangeFactory.createExchange(pattern);
            fail("A MessagingException must be thrown");
        } catch (MessagingException e) {
            // must thow an exception, its ok
        }

    }

    @Test
    public void testCreateInOnlyExchange() throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory.createInOnlyExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOnly(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateInOptionalOutExchange() throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory
                .createInOptionalOutExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOptOut(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateInOutExchange() throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory.createInOutExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOut(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateRobustInOnlyExchange() throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory.createRobustInOnlyExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsRobustInOnly(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateInOnlyExchangeWithContextualFlowInstanceId() throws MessagingException {

        MessageExchange actualMessageExchange = messageExchangeFactory.createInOnlyExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOnly(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateInOptionalOutExchangeWithContextualFlowInstanceId()
            throws MessagingException {

        MessageExchange actualMessageExchange = messageExchangeFactory
                .createInOptionalOutExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOptOut(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateInOutExchangeWithContextualFlowInstanceId() throws MessagingException {

        MessageExchange actualMessageExchange = messageExchangeFactory.createInOutExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOut(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateRobustInOnlyExchangeWithContextualFlowInstanceId()
            throws MessagingException {

        MessageExchange actualMessageExchange = messageExchangeFactory.createRobustInOnlyExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsRobustInOnly(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateInOnlyExchangeWithContextualNullFlowInstanceId()
            throws MessagingException {

        MessageExchange actualMessageExchange = messageExchangeFactory.createInOnlyExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOnly(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateInOptionalOutExchangeWithContextualNullFlowInstanceId()
            throws MessagingException {

        MessageExchange actualMessageExchange = messageExchangeFactory
                .createInOptionalOutExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOptOut(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateInOutExchangeWithContextualNullFlowInstanceId() throws MessagingException {

        MessageExchange actualMessageExchange = messageExchangeFactory.createInOutExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOut(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateRobustInOnlyExchangeWithContextualNullFlowInstanceId()
            throws MessagingException {
        MessageExchange actualMessageExchange = messageExchangeFactory.createRobustInOnlyExchange();

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsRobustInOnly(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromServiceAndOpAndContextualFlowInstanceId()
            throws MessagingException {

        QName serviceName = new QName("targetService");
        QName opName = new QName("targetOperation");

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(serviceName,
                opName);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
        assertGeneratedExchangeUseProvidedServiceAndOp(actualMessageExchange, serviceName, opName);
    }

    @Test
    public void testCreateExchangeFromServiceAndOpAndNullContextualFlowInstanceId()

    throws MessagingException {
        QName serviceName = new QName("targetService");
        QName opName = new QName("targetOperation");

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(serviceName,
                opName);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
        assertGeneratedExchangeUseProvidedServiceAndOp(actualMessageExchange, serviceName, opName);
    }

    @Test
    public void testCreateExchangeFromLegacyInOnlyPatternAndContextualFlowInstanceId()
            throws MessagingException {
        URI pattern = MessageExchangeImpl.IN_ONLY_PATTERN;

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOnly(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateExchangeFromNewInOnlyPatternAndContextualFlowInstanceId()
            throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.IN_ONLY.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOnly(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateExchangeFromNewInOnlyPatternAndNullContextualFlowInstanceId()
            throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.IN_ONLY.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOnly(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromLegacyRobustInOnlyPatternAndContextualFlowInstanceId()
            throws MessagingException {
        URI pattern = MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN;

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsRobustInOnly(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateExchangeFromNewRobustInOnlyPatternAndContextualFlowInstanceId()
            throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.ROBUST_IN_ONLY.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsRobustInOnly(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateExchangeFromNewRobustInOnlyPatternAndNullContextualFlowInstanceId()

    throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.ROBUST_IN_ONLY.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsRobustInOnly(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromLegacyInOptOutPatternAndContextualFlowInstanceId()

    throws MessagingException {
        URI pattern = MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN;

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOptOut(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateExchangeFromNewInOptOutPatternAndContextualFlowInstanceId()
            throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.IN_OPTIONAL_OUT.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOptOut(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateExchangeFromNewInOptOutPatternAndNullContextualFlowInstanceId()
            throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.IN_OPTIONAL_OUT.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOptOut(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromLegacyInOutPatternAndContextualFlowInstanceId()
            throws MessagingException {
        URI pattern = MessageExchangeImpl.IN_OUT_PATTERN;

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOut(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateExchangeFromNewInOutPatternAndContextualFlowInstanceId()
            throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.IN_OUT.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOut(actualMessageExchange);
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    @Test
    public void testCreateExchangeFromNewInOutPatternAndNullContextualFlowInstanceId()
            throws MessagingException {
        URI pattern = AbsItfOperation.MEPPatternConstants.IN_OUT.value();

        MessageExchange actualMessageExchange = messageExchangeFactory.createExchange(pattern);

        assertGeneratedExchangeIsAMessageExchangeDecorator(actualMessageExchange);
        assertGeneratedExchangeIsInOut(actualMessageExchange);
        assertExchangeIdIsPetalsUUID(actualMessageExchange);
    }

    @Test
    public void testCreateExchangeFromNonJBIPatternAndContextualFlowInstanceId() {

        URI pattern = URI.create("http://non/jbi/pattern");

        try {
            messageExchangeFactory.createExchange(pattern);
            fail("A MessagingException must be thrown");
        } catch (MessagingException e) {
            // must thow an exception, its ok
        }

    }

    private void assertGeneratedExchangeIsAMessageExchangeDecorator(
            MessageExchange actualMessageExchange) {
        assertTrue("Returned message exchange isn't a MessageExchangeDecorator",
                actualMessageExchange instanceof MessageExchangeWrapper);
    }

    private void assertExchangeIdIsPetalsUUID(MessageExchange actualMessageExchange) {
        assertIdIsPetalsQualifiedUUID(actualMessageExchange.getExchangeId());
    }

    private void assertGeneratedExchangeUseProvidedServiceAndOp(
            MessageExchange actualMessageExchange, QName serviceName, QName opName) {
        assertEquals("Bad service on created exchange", serviceName,
                actualMessageExchange.getService());
        assertEquals("Bad operation on created exchange", opName,
                actualMessageExchange.getOperation());
    }

    private void assertGeneratedExchangeIsInOnly(MessageExchange actualMessageExchange) {
        assertTrue("Bad pattern on created exchange expected 'InOnly' was '"
                + actualMessageExchange.getClass().getName() + "'",
                actualMessageExchange instanceof InOnly);
    }

    private void assertGeneratedExchangeIsRobustInOnly(MessageExchange actualMessageExchange) {
        assertTrue("Bad pattern on created exchange expected 'RobustInOnly' was '"
                + actualMessageExchange.getClass().getName() + "'",
                actualMessageExchange instanceof RobustInOnly);
    }

    private void assertGeneratedExchangeIsInOptOut(MessageExchange actualMessageExchange) {
        assertTrue("Bad pattern on created exchange expected 'InOptOut' was '"
                + actualMessageExchange.getClass().getName() + "'",
                actualMessageExchange instanceof InOptionalOut);
    }

    private void assertGeneratedExchangeIsInOut(MessageExchange actualMessageExchange) {
        assertTrue("Bad pattern on created exchange expected 'InOut' was '"
                + actualMessageExchange.getClass().getName() + "'",
                actualMessageExchange instanceof InOut);
    }

}
