/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.server;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;

import org.junit.Before;
import org.junit.Test;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.NameController;
import org.ow2.petals.microkernel.api.server.Binding;
import org.ow2.petals.microkernel.api.server.FractalHelper;

/**
 * @author Roland Naudin - EBM WebSourcing
 */
public class FractalHelperTest {

    @Before
    public void setUp() throws Exception {
    }

    @Test
    public void testGetComponentByName() throws NoSuchInterfaceException {
        ContentController contentController = createMock(ContentController.class);
        ContentController subContentController = createMock(ContentController.class);
        Component component = createMock(Component.class);
        NameController nameController = createMock(NameController.class);

        Component[] compos = new Component[] { component };
        Component[] subcompos = new Component[] {};

        expect(contentController.getFcSubComponents()).andReturn(compos);
        expect(component.getFcInterface("content-controller")).andReturn(subContentController);
        expect(subContentController.getFcSubComponents()).andReturn(subcompos);
        expect(component.getFcInterface("name-controller")).andReturn(nameController);
        expect(nameController.getFcName()).andReturn("test");

        replay(contentController);
        replay(subContentController);
        replay(component);
        replay(nameController);

        assertEquals(component,
                FractalHelper.getRecursiveComponentByName(contentController, "test"));

        verify(contentController);
        verify(subContentController);
        verify(component);
        verify(nameController);
    }

    @Test
    public void testGetComponentByNameException() throws NoSuchInterfaceException {
        ContentController contentController = createMock(ContentController.class);
        ContentController subContentController = createMock(ContentController.class);
        Component component = createMock(Component.class);

        Component[] compos = new Component[] { component };
        Component[] subcompos = new Component[] {};

        expect(contentController.getFcSubComponents()).andReturn(compos);
        expect(component.getFcInterface("content-controller")).andReturn(subContentController);
        expect(subContentController.getFcSubComponents()).andReturn(subcompos);
        expect(component.getFcInterface("name-controller")).andThrow(
                new NoSuchInterfaceException("name-controller"));

        replay(contentController);
        replay(subContentController);
        replay(component);

        assertNull(FractalHelper.getRecursiveComponentByName(contentController, "test"));

        verify(contentController);
        verify(subContentController);
        verify(component);
    }

    @Test
    public void testGetLifeCycleControllerByName() throws NoSuchInterfaceException {
        ContentController contentController = createMock(ContentController.class);
        ContentController subContentController = createMock(ContentController.class);
        Component component = createMock(Component.class);
        NameController nameController = createMock(NameController.class);
        LifeCycleController lifeCycleController = createMock(LifeCycleController.class);

        Component[] compos = new Component[] { component };
        Component[] subcompos = new Component[] {};

        expect(contentController.getFcSubComponents()).andReturn(compos);
        expect(component.getFcInterface("content-controller")).andReturn(subContentController);
        expect(subContentController.getFcSubComponents()).andReturn(subcompos);
        expect(component.getFcInterface("name-controller")).andReturn(nameController);
        expect(nameController.getFcName()).andReturn("test");
        expect(component.getFcInterface("lifecycle-controller")).andReturn(lifeCycleController);

        replay(contentController);
        replay(subContentController);
        replay(component);
        replay(nameController);
        replay(lifeCycleController);

        assertEquals(lifeCycleController,
                FractalHelper.getLifeCycleControllerByName(contentController, "test"));

        verify(contentController);
        verify(subContentController);
        verify(component);
        verify(nameController);
        verify(lifeCycleController);

        /**
         * test with a null component name
         */
        reset(contentController);
        reset(subContentController);
        reset(component);
        reset(nameController);
        reset(lifeCycleController);

        expect(contentController.getFcSubComponents()).andReturn(compos);
        expect(component.getFcInterface("content-controller")).andReturn(subContentController);
        expect(subContentController.getFcSubComponents()).andReturn(subcompos);
        expect(component.getFcInterface("name-controller")).andReturn(nameController);
        expect(nameController.getFcName()).andReturn("test");

        replay(contentController);
        replay(subContentController);
        replay(component);
        replay(nameController);
        replay(lifeCycleController);

        assertEquals(null, FractalHelper.getLifeCycleControllerByName(contentController, null));

        verify(contentController);
        verify(subContentController);
        verify(component);
        verify(nameController);
        verify(lifeCycleController);
    }

    @Test
    public void testStartComponent() throws NoSuchInterfaceException, IllegalLifeCycleException {
        LifeCycleController lifeCycleController = createMock(LifeCycleController.class);
        Component component = createMock(Component.class);

        expect(component.getFcInterface("lifecycle-controller")).andReturn(lifeCycleController);

        expect(lifeCycleController.getFcState()).andReturn(LifeCycleController.STOPPED);
        lifeCycleController.startFc();

        replay(component);
        replay(lifeCycleController);

        assertTrue(FractalHelper.startComponent(component));

        verify(component);
        verify(lifeCycleController);

        /*
         * the component is not stopped, the method return false
         */
        reset(component);
        reset(lifeCycleController);

        expect(component.getFcInterface("lifecycle-controller")).andReturn(lifeCycleController);

        expect(lifeCycleController.getFcState()).andReturn(LifeCycleController.STARTED);

        replay(component);
        replay(lifeCycleController);

        assertFalse(FractalHelper.startComponent(component));

        verify(component);
    }

    @Test
    public void testStopComponent() throws NoSuchInterfaceException, IllegalLifeCycleException {
        LifeCycleController lifeCycleController = createMock(LifeCycleController.class);
        Component component = createMock(Component.class);

        expect(component.getFcInterface("lifecycle-controller")).andReturn(lifeCycleController);
        expect(lifeCycleController.getFcState()).andReturn(LifeCycleController.STARTED);
        lifeCycleController.stopFc();

        replay(component);
        replay(lifeCycleController);

        assertTrue(FractalHelper.stopComponent(component));

        verify(component);
        verify(lifeCycleController);

        /*
         * the component is not started, the method return false
         */
        reset(component);
        reset(lifeCycleController);

        expect(component.getFcInterface("lifecycle-controller")).andReturn(lifeCycleController);
        expect(lifeCycleController.getFcState()).andReturn(LifeCycleController.STOPPED);

        replay(component);
        replay(lifeCycleController);

        assertFalse(FractalHelper.stopComponent(component));

        verify(component);
        verify(lifeCycleController);
    }

    @Test
    public void testAddComponent() throws NoSuchInterfaceException, IllegalContentException,
            IllegalLifeCycleException, IllegalBindingException, ADLException {
        ContentController contentController = createMock(ContentController.class);
        Component newComponent = createMock(Component.class);
        Component parentComponent = createMock(Component.class);
        BindingController bindingController = createMock(BindingController.class);
        Binding binding = createMock(Binding.class);
        Interface serverInterface = createMock(Interface.class);

        ArrayList<Binding> listOfBindings = new ArrayList<Binding>();
        listOfBindings.add(binding);

        expect(parentComponent.getFcInterface("content-controller")).andReturn(contentController);
        contentController.addFcSubComponent(newComponent);
        expect(newComponent.getFcInterface("binding-controller")).andReturn(bindingController);
        expect(binding.getClientInterfaceName()).andReturn("clientInterface");
        expect(binding.getServerInterface()).andReturn(serverInterface);
        bindingController.bindFc("clientInterface", serverInterface);

        replay(contentController);
        replay(newComponent);
        replay(parentComponent);
        replay(bindingController);
        replay(binding);
        replay(serverInterface);

        FractalHelper.addComponent(newComponent, parentComponent, listOfBindings);

        verify(contentController);
        verify(newComponent);
        verify(parentComponent);
        verify(bindingController);
        verify(binding);
        verify(serverInterface);
    }

}
