/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.classloader;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.fail;

import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLStreamHandlerFactory;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import org.easymock.classextension.EasyMock;
import org.junit.Before;
import org.junit.Test;

/**
 * Test the petals class loader.
 * @author ddesjardins - EBM WebSourcing
 */
public class PetalsClassLoaderTest {

    /**
     * PetalsClassLoader to test
     */
    private PetalsClassLoader petalsClassLoader;

    @Before
    public void setUp() throws IOException, URISyntaxException {
        List<URL> baseUrls = new ArrayList<URL>();
        List<String> classpathLocations = new ArrayList<String>();

        final URL junitUrl = Thread.currentThread().getContextClassLoader()
                .getResource("junit-3.8.jar");
        final File junitFile = new File(junitUrl.toURI());
        final File testClassesDir = junitFile.getParentFile();
        final File targetDir = testClassesDir.getParentFile();
        final File baseDir = targetDir.getParentFile();

        baseUrls.add(baseDir.toURI().toURL());
        baseUrls.add(testClassesDir.toURI().toURL());
        baseUrls.add(targetDir.toURI().toURL());
        baseUrls.add(junitUrl);

        petalsClassLoader = new PetalsClassLoader(baseUrls.toArray(new URL[0]), classpathLocations);
        petalsClassLoader.setParentFirst(false);
    }

    /**
     * Test to get the classpath
     * 
     */
    @Test
    public void testGetClasspath() {
        petalsClassLoader.getClasspath();

    }

    /**
     * Test to load a class from a directory
     * 
     */
    @Test
    public void testLoadClassFromDirectory() {
        try {

            Class class1 = petalsClassLoader.loadClass(
                            "org.ow2.petals.microkernel.system.classloader.ComponentClassLoaderTest",
                            false);
            class1.getName();

        } catch (ClassNotFoundException e) {

            fail("Class not found : org.ow2.petals.microkernel.system.classloader.ComponentClassLoaderTest");
        }
    }

    /**
     * Test to load a class from a jar
     * 
     */
    @Test
    public void testLoadClassFromJar() {
        try {

            Class class1 = petalsClassLoader.loadClass("junit.framework.TestCase", false);
            class1.getName();

        } catch (ClassNotFoundException e) {

            fail("Class not found : junit.framework.TestCase");
        }
    }

    /**
     * Test to get a resource from a jar
     * 
     */
    @Test
    public void testGetResourceFromAJar() {
        URL url = petalsClassLoader.getResource("META-INF/MANIFEST.MF");
        if (url != null) {
        } else {
            fail("testGetResource - resource not found");
        }
    }

    /**
     * Test to get a resource from a directory
     * 
     */
    @Test
    public void testGetResourceFromADirectory() {
        URL url = petalsClassLoader.getResource("src/ResourceToLoad");
        if (url != null) {
        } else {
            fail("testGetResource - resource not found");
        }
    }

    /**
     * Test to get a resource that does not exist
     * 
     */
    @Test
    public void testGetResourceThatDoesNotExist() {
        final URL url = petalsClassLoader.getResource("test");
        assertNull("testGetResource - resource found problem", url);
    }

    /**
     * Test to get a resource as a stream from a directory
     * 
     * @throws IOException
     */
    @Test
    public void testGetResourceAsStream() throws IOException {
        InputStream inputStream = petalsClassLoader.getResourceAsStream("src/ResourceToLoad");
        assertNotNull("testGetResourceAsStream - stream not found", inputStream);

        DataInputStream dis = new DataInputStream(inputStream);
        byte[] content = new byte[dis.available()];
        dis.readFully(content);
    }

    /**
     * Test to get a resource as a stream from a jar
     * 
     * @throws IOException
     */
    @Test
    public void testGetResourceAsStreamFromAJar() throws IOException {
        InputStream inputStream = petalsClassLoader.getResourceAsStream("META-INF/MANIFEST.MF");
        assertNotNull("testGetResourceAsStreamFromAJar - stream not found", inputStream);

        DataInputStream dis = new DataInputStream(inputStream);
        byte[] content = new byte[dis.available()];
        dis.readFully(content);
    }

    /**
     * Get a resource as stream where the resource name contains spaces.
     */
    @Test
    public void testGetResourceWithSpaceAsStrem() throws IOException {
        final InputStream inputStream = petalsClassLoader
                .getResourceAsStream("resource with spaces.txt");
        assertNotNull("testGetResourceWithSpaceAsStrem - stream not found", inputStream);

        DataInputStream dis = new DataInputStream(inputStream);
        byte[] content = new byte[dis.available()];
        dis.readFully(content);

    }

    /**
     * Test to get resources
     * 
     * @throws IOException
     */
    @Test
    public void testGetResources() throws IOException {
        Enumeration<URL> enumeration = petalsClassLoader.getResources("src/ResourceToLoad");
        if (enumeration.hasMoreElements()) {
        } else {
            fail("testGetResource - resource not found");
        }
    }

    @Test
    public void testConstructor() throws Throwable {
        URLStreamHandlerFactory urlStreamHandlerFactory = EasyMock
                .createMock(URLStreamHandlerFactory.class);
        PetalsClassLoader petalsClassLoader = new PetalsClassLoader(new URL[0],
                new ArrayList<String>(), this.getClass().getClassLoader(), urlStreamHandlerFactory);
        assertEquals(petalsClassLoader.getParent(), this.getClass().getClassLoader());
        petalsClassLoader.close();
        assertEquals(petalsClassLoader.getBases().length, 0);
    }

}
